@@ -30,6 +30,7 @@
 #include <set>
 #include <cstdio>
 #include <unistd.h>
+#include <getopt.h>
 #include <cerrno>
 #include <cctype>
 #include <sys/stat.h>
@@ -59,6 +60,7 @@ set<string> menufiles_processed;
 
 translateinfo *transinfo;
 configinfo     config;
+bool is_root;
 
 /** Try to open a directory. Throws dir_error_read if failed, and a DIR*
  * descriptor if succeeded.
@@ -347,7 +349,7 @@ translateinfo::translateinfo(const std::string &filename)
   try {
     i = new parsestream(filename);
 
-    Regex ident("[a-zA-Z_][a-zA-Z0-9_]*");
+    Regex ident("[[:alpha:]][[:alnum:]_]*");
     /*Translation here and below refer to the file 
       /etc/menu-methods/translate_menus that allow to rename and reorganize
       menu entries automatically. It does not refer to the localisation
@@ -632,9 +634,8 @@ void run_menumethod(string methodname, const vector<string> &menudata)
 /** Run a directory full of menu methods */
 void run_menumethoddir(const string &dirname, const vector<string> &menudata)
 {
-  struct stat st;
   struct dirent *entry;
-  char *s, tmp[MAX_LINE];
+  char *s;
 
   config.report(String::compose(_("Running menu-methods in %1."), dirname), configinfo::report_verbose);
   DIR *dir = open_dir_check(dirname);
@@ -648,17 +649,11 @@ void run_menumethoddir(const string &dirname, const vector<string> &menudata)
     }
     if (*s != '\0')
       continue;
+    
+    string method = dirname + entry->d_name;
 
-    sprintf(tmp, "%s%s", dirname.c_str(), entry->d_name);
-    int r = stat(tmp, &st);
-
-    // Do we have execute permissions? 
-    if ((!r) &&
-        (((st.st_mode & S_IXOTH) || 
-          ((st.st_mode & S_IXGRP) && st.st_gid == getegid ()) || 
-          ((st.st_mode & S_IXUSR) && st.st_uid == geteuid ())) && 
-         (S_ISREG(st.st_mode) || S_ISLNK(st.st_mode))))
-        run_menumethod(tmp, menudata);
+    if (executable(method))
+        run_menumethod(method, menudata);
   }
   closedir (dir);
 }
@@ -670,7 +665,7 @@ int create_lock()
   int fd = true;
   char buf[64];
 
-  if (!getuid()) {
+  if (is_root) {
     fd = open(UPMEN_LOCKFILE,O_WRONLY|O_CREAT,00644);
 
     if (flock(fd,LOCK_EX|LOCK_NB)) {
@@ -698,7 +693,7 @@ int create_lock()
 /** Try to remove update-menus lock */
 void remove_lock()
 {
-  if (!getuid()){
+  if (is_root) {
     if (unlink(UPMEN_LOCKFILE))
       config.report(String::compose(_("Cannot remove lockfile %1."), UPMEN_LOCKFILE),
           configinfo::report_normal);
@@ -714,7 +709,7 @@ int check_dpkglock()
 {
   int fd;
   struct flock fl;
-  if (getuid())
+  if (!is_root)
   {
     config.report(_("Update-menus is run by user."), configinfo::report_verbose);
     return 0;
@@ -834,63 +829,69 @@ void wait_dpkg(string &stdoutfile)
 void usage(ostream &c)
 {
       c <<
-          /* This is the update-menus --help message*/
-          _("update-menus: update the various window-manager config files (and\n"
-              "  dwww, and pdmenu) Usage: update-menus [options] \n"
-              "    -d              Output debugging messages.\n"
-              "    -v              Be verbose about what is going on.\n"
-              "    -h, --help      This message.\n"
-              "    --menufilesdir  <dir> Add <dir> to the lists of menu directories to search.\n"
-              "    --menumethod    <method> Run only the menu method <method>.\n"
-              "    --nodefaultdirs Disables the use of all the standard menu directories.\n"
-              "    --stdout        Output menu list in format suitable for piping to\n"
-              "                    install-menu.\n")
-          /* This is the end of the update-menus --help message*/
-          << _(  "    --version       Output version information and exit.\n"  );
+          _( /* This is the update-menus --help message*/
+  "Usage: update-menus [options] \n"
+  "Gather packages data from the menu database and generate menus for\n"
+  "all programs providing menu-methods, usually window-managers.\n"
+  "  -d                     Output debugging messages.\n"
+  "  -v                     Be verbose about what is going on.\n"
+  "  -h, --help             This message.\n"
+  "  --menufilesdir=<dir>   Add <dir> to the lists of menu directories to search.\n"
+  "  --menumethod=<method>  Run only the menu method <method>.\n"
+  "  --nodefaultdirs        Disable the use of all the standard menu directories.\n"
+  "  --stdout               Output menu list in format suitable for piping to\n"
+  "                         install-menu.\n")
+       << _(  /* This is the end of the update-menus --help message*/
+  "  --version              Output version information and exit.\n"  );
 }
 
+struct option long_options[] = { 
+  { "debug", no_argument, NULL, 'd' }, 
+  { "verbose", no_argument, NULL, 'v' }, 
+  { "help", no_argument, NULL, 'h' }, 
+  { "menufilesdir", required_argument, NULL, 'f'},
+  { "menumethod", required_argument, NULL, 'm'},
+  { "nodefaultdirs", no_argument, NULL, 'n'},
+  { "stdout", no_argument, NULL, 's'},
+  { "version", no_argument, NULL, 'V'},
+  { NULL, 0, NULL, 0 } };
+
+
 /** Parse commandline parameters */
-void parse_params(char **argv)
+void parse_params(int argc, char **argv)
 {
-  while (*(++argv))
+  while(1)
   {
-    if (string("-v") == *argv) {
-      config.set_verbosity(configinfo::report_verbose);
-      continue;
-    } else if (string("-d") == *argv) {
-      config.set_verbosity(configinfo::report_debug);
-      continue;
-    } else if (string("--nodefaultdirs") == *argv) {
+    int c = getopt_long (argc, argv, "hvd", long_options, NULL);
+    if (c == -1) 
+      break;
+    switch(c)
+    {
+    case 'v':
+        config.set_verbosity(configinfo::report_verbose);
+      break;
+    case 'd':
+        config.set_verbosity(configinfo::report_verbose);
+      break;
+    case 'n':
       config.usedefaultmenufilesdirs = false;
-      continue;
-    } else if (string("--stdout") == *argv) {
+      break;
+    case 's':
       config.onlyoutput_to_stdout = true;
-      continue;
-    } else if (string("--menufilesdir") == *argv || string("--menufiledir") == *argv) {
-      argv++;
-      if(*argv) {
-          config.menufilesdir.push_back(*argv);
-      } else {
-        cerr<< _("Directory is expected after --menufilesdir option.\n");
-        throw informed_fatal();
-      }
-      continue;
-    } else if (string("--menumethod") == *argv) {
-      argv++;
-      if(*argv) {
-          config.menumethod = *argv;
-      } else {
-        cerr << _("Filename is expected after --menumethod option.\n");
-        throw informed_fatal();
-      }
-      continue;
-    } else if (string("--version") == *argv) {
-        cout << "update-menus "VERSION << std::endl;
-        exit(0);
-    } else if (string("-h") == *argv || string("--help") == *argv) {  
+      break;
+    case 'f':
+      config.menufilesdir.push_back(optarg);
+      break;
+    case 'm':
+      config.menumethod = optarg;
+      break;
+    case 'V':
+      cout << "update-menus "VERSION << std::endl;
+      exit(0);
+    case 'h':
       usage(cout);
       exit(0);
-    } else {
+    default: 
       usage(cerr);
       exit(1);
     }
@@ -900,7 +901,7 @@ void parse_params(char **argv)
 /** Read users configuration file */
 void read_userconfiginfo()
 {
-  if (getuid()) {
+  if (!is_root) {
     try {
       config.read_file(string(home_dir)+"/"+USERCONFIG);
     } catch(ferror_open d) { };
@@ -920,7 +921,7 @@ void read_rootconfiginfo()
 /** Read users translate information */
 void read_usertranslateinfo()
 {
-  if (getuid()) {
+  if (!is_root) {
     try {
       transinfo = new translateinfo(string(home_dir)+"/"+USERTRANSLATE);
     } catch(ferror_open d) { };
@@ -950,6 +951,7 @@ void read_homedirectory()
 
 int main (int argc, char **argv)
 {
+  is_root = (getuid() == 0);
   read_homedirectory();
 
   vector<string> menudata;
@@ -962,7 +964,7 @@ int main (int argc, char **argv)
 
   try {
     read_rootconfiginfo();
-    parse_params(argv);
+    parse_params(argc, argv);
     wait_dpkg(stdoutfile);
     if(!stdoutfile.empty()) {
       close(1);
@@ -977,7 +979,7 @@ int main (int argc, char **argv)
     read_roottranslateinfo();
 
     if (config.usedefaultmenufilesdirs) {
-      if (getuid())
+      if (!is_root)
           config.menufilesdir.push_back(string(home_dir)+"/"+USERMENUS);
       config.menufilesdir.push_back(CONFIGMENUS);
       config.menufilesdir.push_back(PACKAGEMENUSLIB);
@@ -992,9 +994,13 @@ int main (int argc, char **argv)
               cout << *i;
 
     } else if (!config.menumethod.empty()) {
-      run_menumethod(config.menumethod, menudata);
+      if (executable(config.menumethod))
+        run_menumethod(config.menumethod, menudata);
+      else
+        config.report(String::compose(_("Script %1 could not be executed."),
+              config.menumethod), configinfo::report_quiet);
     } else {
-      if (getuid()) {
+      if (!is_root) {
         try {
           run_menumethoddir(string(home_dir)+"/"+USERMETHODS, menudata);
         }
